package com.biganto.visual.roompark.domain.use_case

import com.biganto.visual.roompark.data.repository.db.requrey.model.DealEntity
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.contract.DealContract
import com.biganto.visual.roompark.domain.model.DealModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 24.09.2019.
 */

class DealseUseCase @Inject constructor(
    private val contract: DealContract,
    private val authContract: AuthContract
) {

    fun prefetchDeal(): Observable<List<DealModel>> =
        authContract.currentUser()
            .map {user ->
                user.deals?.asSequence()
                ?.map { it as DealEntity }
                ?.filterNotNull()?.toList()
            }
            .map { fromEntity(it, ::fromEntity) }

    fun getDeals(): Observable<List<DealModel>> =
        authContract.currentUser()
            .flatMap(contract::getDealsApi)
            .map { fromEntity(it, ::fromEntity) }

    fun setDealRead(id:String) = contract.setDealRead(id)


}