package com.biganto.visual.roompark.presentation.screen.favorites

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.FavoritesInteractor
import com.biganto.visual.roompark.domain.interactor.ToursInteractor
import com.biganto.visual.roompark.domain.model.EstateModel
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class FavoritesScreenPresenter @Inject constructor(
    private val interactor: FavoritesInteractor,
    private val toursInteractor: ToursInteractor
)
    : BigantoBasePresenter<FavoritesScreen, FavoritesScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> FavoritesScreenViewState.SomeError(e) }

    private val restoreModel = RestoreModel(mutableListOf())

    override fun detachView() {
        super.detachView()
        restoreStateObservable.accept(FavoritesScreenViewState.RestoreView(restoreModel))
    }

    override fun bindIntents() {

        val prefetchCards = interactor.cachedFavorites()
            .doOnNext { restoreModel.list = it.toMutableList() }
            .map<FavoritesScreenViewState> { FavoritesScreenViewState.FavoriteEstatesLoaded(it) }
            .onErrorReturn (::parseError)

        val invalidateData = interactor.getFavoritesForCurrentUser()
            .doOnNext { restoreModel.list = it.toMutableList() }
            .map<FavoritesScreenViewState> { FavoritesScreenViewState.FavoriteEstatesLoaded(it) }
            .onErrorReturn (::parseError)


        val onStartTours = intent(FavoritesScreen::tourCardClicked)
            .flatMap {estate -> toursInteractor.getEstateTourList(estate)
                .map<FavoritesScreenViewState> { FavoritesScreenViewState.ToursLoaded(it) }
                .onErrorReturn (::parseError)
            }


        val state = Observable.mergeDelayError(
            arrayListOf(
                restoreStateObservable,
                prefetchCards,
                invalidateData,
                onStartTours
            )
        )
            .doOnError { Timber.e(it) }
            .onErrorReturn(::parseError)
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(
            state.cast(FavoritesScreenViewState::class.java),
            FavoritesScreen::render
        )
    }

}

data class RestoreModel(
    var list:MutableList<EstateModel>
)